<?php

namespace Toplist\Toplist\Providers;

use Illuminate\Support\ServiceProvider;

class ToplistServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'toplist');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'toplist');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerToplist();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerToplist()
    {
        $this->app->bind('toplist', function($app) {
            return new Toplist($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Toplist', 'Lavalite\Toplist\Facades\Toplist');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('toplist.toplist', function ($app) {
            return $this->app->make('Toplist\Toplist\Toplist');
        });

                // Bind Toplist to repository
        $this->app->bind(
            'Toplist\Toplist\Interfaces\ToplistRepositoryInterface',
            \Toplist\Toplist\Repositories\Eloquent\ToplistRepository::class
        );

        $this->app->register(\Toplist\Toplist\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Toplist\Toplist\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and toplist's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/toplist.php', 'toplist'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeToplist::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['toplist.toplist'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('toplist/toplist.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/toplist')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/toplist')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
